"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var XPath = require("xpath");
var Constants_1 = require("./Constants");
var Regexp_1 = require("./Regexp");
var Utils_1 = require("./Utils");
function addCheckpoint(document, element, options) {
    var _a = __assign({ count: 0, level: 0, nodeLimit: Constants_1.DefaultNodeLimit }, options), count = _a.count, level = _a.level, nodeLimit = _a.nodeLimit;
    if (element.firstChild && element.firstChild.nodeType === Constants_1.NodeTypes.TEXT_NODE)
        element.replaceChild(document.createTextNode("" + (element.firstChild.nodeValue || "") + Constants_1.CheckpointPrefix + count + Constants_1.CheckpointSuffix), element.firstChild);
    else
        element.nodeValue = "" + Constants_1.CheckpointPrefix + count + Constants_1.CheckpointSuffix;
    count++;
    if (count < nodeLimit)
        for (var index = 0; index < element.childNodes.length && count < nodeLimit; index++) {
            var node = element.childNodes.item(index);
            if (node.nodeType !== 1)
                continue;
            count = addCheckpoint(document, node, { nodeLimit: nodeLimit, count: count, level: level + 1 });
        }
    if (element.nextSibling && element.nextSibling.nodeType === Constants_1.NodeTypes.TEXT_NODE)
        element.parentNode.replaceChild(document.createTextNode("" + (element.nextSibling.nodeValue || "") + Constants_1.CheckpointPrefix + count + Constants_1.CheckpointSuffix), element.nextSibling);
    else if (element.parentNode)
        element.parentNode.appendChild(document.createTextNode("" + Constants_1.CheckpointPrefix + count + Constants_1.CheckpointSuffix));
    count++;
    return count;
}
exports.addCheckpoint = addCheckpoint;
;
function deleteQuotationTags(document, element, quotationCheckpoints, options) {
    var _a;
    var isTagInQuotation = true;
    var _b = __assign({ count: 0, level: 0, nodeLimit: Constants_1.DefaultNodeLimit }, options), count = _b.count, level = _b.level, nodeLimit = _b.nodeLimit;
    if (quotationCheckpoints[count]) {
        if (element.firstChild && element.firstChild.nodeType === Constants_1.NodeTypes.TEXT_NODE)
            element.replaceChild(document.createTextNode(""), element.firstChild);
        else
            element.nodeValue = "";
    }
    else {
        isTagInQuotation = false;
    }
    count++;
    var preserveTable = !isTagInQuotation && element.nodeName === 'table';
    var quotationChildren = new Array();
    if (count < nodeLimit && !preserveTable)
        for (var index = 0; index < element.childNodes.length && count < nodeLimit; index++) {
            var node = element.childNodes.item(index);
            if (node.nodeType !== Constants_1.NodeTypes.ELEMENT_NODE)
                continue;
            var isChildTagInQuotation = void 0;
            (_a = deleteQuotationTags(document, node, quotationCheckpoints, { nodeLimit: nodeLimit, count: count, level: level + 1 }), count = _a.count, isChildTagInQuotation = _a.isTagInQuotation);
            if (!isChildTagInQuotation)
                continue;
            quotationChildren.push(node);
        }
    if (quotationCheckpoints[count]) {
        if (element.nextSibling && element.nextSibling.nodeType === Constants_1.NodeTypes.TEXT_NODE)
            element.parentNode.replaceChild(document.createTextNode(""), element.nextSibling);
    }
    else {
        isTagInQuotation = false;
    }
    count++;
    if (!isTagInQuotation)
        for (var _i = 0, quotationChildren_1 = quotationChildren; _i < quotationChildren_1.length; _i++) {
            var node = quotationChildren_1[_i];
            node.parentNode.removeChild(node);
        }
    return {
        count: count,
        isTagInQuotation: isTagInQuotation
    };
}
exports.deleteQuotationTags = deleteQuotationTags;
function shouldNotRemoveQuote(node, options) {
    return Boolean(options && options.onlyRemoveEmptyBlocks && node.textContent.trim() !== '');
}
function cutGmailQuote(document, options) {
    var gmailQuote = XPath.select("//*[contains(@class, 'gmail_quote')]", document, true);
    if (!gmailQuote || (gmailQuote.textContent && Utils_1.matchStart(gmailQuote.textContent, Regexp_1.ForwardRegexp)) || shouldNotRemoveQuote(gmailQuote, options))
        return false;
    gmailQuote.parentNode.removeChild(gmailQuote);
    return true;
}
exports.cutGmailQuote = cutGmailQuote;
;
function cutMicrosoftQuote(document, options) {
    var splitter = XPath.select("//*[local-name(.)='div' and @style='border:none;" +
        "border-top:solid #B5C4DF 1.0pt;" +
        "padding:3.0pt 0cm 0cm 0cm']|" +
        "//*[local-name(.)='div' and @style='padding-top: 5px; " +
        "border-top-color: rgb(229, 229, 229); " +
        "border-top-width: 1px; border-top-style: solid;']", document, true);
    if (splitter) {
        if (splitter.parentElement && splitter === splitter.parentElement.children[0])
            splitter = splitter.parentNode;
    }
    else {
        splitter = XPath.select("//*[local-name(.)='div']" +
            "/*[local-name(.)='div' and @class='MsoNormal' and @align='center' and @style='text-align:center']" +
            "/*[local-name(.)='font']" +
            "/*[local-name(.)='span']" +
            "/*[local-name(.)='hr' and @size='3' and @width='100%' and @align='center' and @tabindex='-1']", document, true);
        if (splitter) {
            if (splitter.parentNode)
                splitter = splitter.parentNode;
            if (splitter.parentNode)
                splitter = splitter.parentNode;
            if (splitter.parentNode)
                splitter = splitter.parentNode;
            if (splitter.parentNode)
                splitter = splitter.parentNode;
        }
    }
    if (!splitter || shouldNotRemoveQuote(splitter, options))
        return false;
    while (splitter.nextSibling)
        splitter.parentNode.removeChild(splitter.nextSibling);
    splitter.parentNode.removeChild(splitter);
    return true;
}
exports.cutMicrosoftQuote = cutMicrosoftQuote;
;
function cutZimbraQuote(document, options) {
    var splitter = XPath.select("//*[local-name(.)='hr' and @data-marker=\"__DIVIDER__\"]", document, true);
    if (!splitter || shouldNotRemoveQuote(splitter, options))
        return false;
    splitter.parentNode.removeChild(splitter);
    return true;
}
exports.cutZimbraQuote = cutZimbraQuote;
;
function cutById(document, options) {
    var found = false;
    for (var _i = 0, QuoteIds_1 = Constants_1.QuoteIds; _i < QuoteIds_1.length; _i++) {
        var quoteId = QuoteIds_1[_i];
        var quote = XPath.select("//*[@id=\"" + quoteId + "\"]", document, true);
        if (!quote || shouldNotRemoveQuote(quote, options))
            continue;
        found = true;
        quote.parentNode.removeChild(quote);
    }
    return found;
}
exports.cutById = cutById;
;
function cutBlockquote(document, options) {
    var quote = XPath.select("(.//*[local-name(.)='blockquote'])" +
        "[not(@class=\"gmail_quote\") and not(ancestor::blockquote)]" +
        "[last()]", document, true);
    if (!quote || shouldNotRemoveQuote(quote, options))
        return false;
    quote.parentNode.removeChild(quote);
    return true;
}
exports.cutBlockquote = cutBlockquote;
;
//# sourceMappingURL=HtmlQuotations.js.map