"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var Cheerio = require("cheerio");
var XmlDom = require("xmldom");
var Constants_1 = require("./Constants");
var HtmlQuotations_1 = require("./HtmlQuotations");
var Regexp_1 = require("./Regexp");
var Utils_1 = require("./Utils");
var xmlDomParser = new XmlDom.DOMParser({ errorHandler: { warning: function () { }, error: function () { }, fatalError: function (error) { throw error; } } });
var xmlDomSerializer = new XmlDom.XMLSerializer();
function extractFromPlain(messageBody, options) {
    if (!messageBody || !messageBody.trim())
        return { body: messageBody, didFindQuote: false };
    var maxLinesCount = __assign({ maxLinesCount: Constants_1.DefaultMaxLinesCount }, options).maxLinesCount;
    var delimiter = Utils_1.findDelimiter(messageBody);
    messageBody = preprocess(messageBody, delimiter);
    var lines = Utils_1.splitLines(messageBody).slice(0, maxLinesCount);
    var markers = markMessageLines(lines);
    var _a = processMarkedLines(lines, markers), wereLinesDeleted = _a.wereLinesDeleted, lastMessageLines = _a.lastMessageLines;
    messageBody = lastMessageLines.join(delimiter);
    messageBody = postProcess(messageBody);
    return { body: messageBody, didFindQuote: wereLinesDeleted };
}
exports.extractFromPlain = extractFromPlain;
function extractFromHtml(messageBody, options) {
    if (!messageBody || !messageBody.trim())
        return { body: messageBody, didFindQuote: false };
    var _a = __assign({ nodeLimit: Constants_1.DefaultNodeLimit, maxLinesCount: Constants_1.DefaultMaxLinesCount }, options), nodeLimit = _a.nodeLimit, maxLinesCount = _a.maxLinesCount;
    messageBody = messageBody.replace(/\r\n/g, " ").replace(/\n/g, " ");
    messageBody = Utils_1.normalizeHtmlDocument(messageBody);
    var document = loadHtmlAndFix(messageBody);
    var xmlDocument = xmlDomParser.parseFromString(document.xml());
    if (xmlDocument.lastChild && xmlDocument.lastChild.nodeValue)
        xmlDocument.removeChild(xmlDocument.lastChild);
    var xmlDocumentCopy = xmlDocument.cloneNode(true);
    var numberOfCheckpoints = HtmlQuotations_1.addCheckpoint(xmlDocument, xmlDocument, { nodeLimit: nodeLimit });
    if (numberOfCheckpoints >= nodeLimit)
        return { body: messageBody, didFindQuote: false, isTooLong: true };
    var extractQuoteHtml = extractQuoteHtmlViaMarkers(numberOfCheckpoints, xmlDocument, { ignoreBlockTags: false, maxLinesCount: maxLinesCount });
    if (extractQuoteHtml.error)
        return { body: messageBody, didFindQuote: false, isTooLong: true };
    if (!extractQuoteHtml.quoteWasFound)
        extractQuoteHtml = extractQuoteHtmlViaMarkers(numberOfCheckpoints, xmlDocument, { ignoreBlockTags: true, maxLinesCount: maxLinesCount });
    if (extractQuoteHtml.quoteWasFound) {
        HtmlQuotations_1.deleteQuotationTags(xmlDocument, xmlDocumentCopy, extractQuoteHtml.quotationCheckpoints, { nodeLimit: nodeLimit });
        if (xmlDocumentCopy.nodeType === Constants_1.NodeTypes.DOCUMENT_NODE && !xmlDocumentCopy.documentElement)
            xmlDocumentCopy.documentElement = xmlDocumentCopy.childNodes[0];
        cutQuotation(xmlDocumentCopy, { onlyRemoveEmptyBlocks: true });
        return {
            body: xmlDomSerializer.serializeToString(xmlDocumentCopy, true),
            didFindQuote: true
        };
    }
    var cutQuotations = cutQuotation(xmlDocumentCopy);
    if (cutQuotations)
        return { body: xmlDomSerializer.serializeToString(xmlDocumentCopy, true), didFindQuote: true };
    return { body: messageBody, didFindQuote: false };
}
exports.extractFromHtml = extractFromHtml;
function cutQuotation(xmlDocument, options) {
    return HtmlQuotations_1.cutGmailQuote(xmlDocument, options)
        || HtmlQuotations_1.cutZimbraQuote(xmlDocument, options)
        || HtmlQuotations_1.cutBlockquote(xmlDocument, options)
        || HtmlQuotations_1.cutMicrosoftQuote(xmlDocument, options)
        || HtmlQuotations_1.cutById(xmlDocument, options);
}
function extractQuoteHtmlViaMarkers(numberOfCheckpoints, xmlDocument, options) {
    var _a = __assign({ ignoreBlockTags: false, maxLinesCount: Constants_1.DefaultMaxLinesCount }, options), ignoreBlockTags = _a.ignoreBlockTags, maxLinesCount = _a.maxLinesCount;
    var messagePlainText = preprocess(Utils_1.elementToText(xmlDocument, { ignoreBlockTags: ignoreBlockTags }), "\n", Constants_1.ContentTypeTextPlain);
    var lines = Utils_1.splitLines(messagePlainText);
    if (lines.length > maxLinesCount)
        return { error: 'Message too big' };
    var lineCheckpoints = lines.map(function (line) {
        var match = line.match(new RegExp(Regexp_1.CheckPointRegexp.source, "g"));
        return match
            ? match.map(function (matchPart) { return parseInt(matchPart.slice(4, -4), 10); })
            : new Array();
    });
    lines = lines.map(function (line) { return line.replace(new RegExp(Regexp_1.CheckPointRegexp.source, "g"), ""); });
    var markers = markMessageLines(lines);
    var _b = processMarkedLines(lines, markers), wereLinesDeleted = _b.wereLinesDeleted, firstDeletedLine = _b.firstDeletedLine, lastDeletedLine = _b.lastDeletedLine;
    var quotationCheckpoints = new Array(numberOfCheckpoints);
    if (wereLinesDeleted)
        for (var index = firstDeletedLine; index <= lastDeletedLine; index++)
            for (var _i = 0, _c = lineCheckpoints[index]; _i < _c.length; _i++) {
                var checkpoint = _c[_i];
                quotationCheckpoints[checkpoint] = true;
            }
    return { quoteWasFound: wereLinesDeleted, quotationCheckpoints: quotationCheckpoints };
}
function preprocess(messageBody, delimiter, contentType) {
    if (contentType === void 0) { contentType = Constants_1.ContentTypeTextPlain; }
    messageBody = messageBody.replace(new RegExp(Regexp_1.LinkRegexp.source, "g"), function (match, link, offset, str) {
        var newLineIndex = str.substring(offset).indexOf("\n");
        return str[newLineIndex + 1] === ">" ? match : "@@" + link + "@@";
    });
    if (contentType !== Constants_1.ContentTypeTextPlain)
        return messageBody;
    messageBody = messageBody.replace(new RegExp(Regexp_1.OnDateSomebodyWroteRegexp.source, "g"), function (match) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var offset = args.filter(function (a) { return isFinite(a); })[0];
        var str = args[args.length - 1];
        return offset > 0 && str[offset - 1] !== "\n"
            ? delimiter + match
            : match;
    });
    return messageBody;
}
exports.preprocess = preprocess;
function markMessageLines(lines) {
    var markers = new Array(lines.length);
    var index = 0;
    while (index < lines.length) {
        var line = lines[index];
        if (!line) {
            markers[index] = "e";
        }
        else if (Utils_1.matchStart(line, Regexp_1.QuotePatternRegexp)) {
            markers[index] = "m";
        }
        else if (Utils_1.matchStart(line, Regexp_1.ForwardRegexp)) {
            markers[index] = "f";
        }
        else {
            var splitterMatch = isSplitter(lines.slice(index, index + Constants_1.SplitterMaxLines).join("\n"));
            if (!splitterMatch) {
                markers[index] = "t";
            }
            else {
                var splitterLines = Utils_1.splitLines(splitterMatch[0]);
                for (var splitterIndex = 0; splitterIndex < splitterLines.length; splitterIndex++)
                    markers[index + splitterIndex] = "s";
                index += splitterLines.length - 1;
            }
        }
        index++;
    }
    return markers.join("");
}
exports.markMessageLines = markMessageLines;
function processMarkedLines(lines, markers) {
    var result = {
        lastMessageLines: lines,
        wereLinesDeleted: false,
        firstDeletedLine: -1,
        lastDeletedLine: -1
    };
    if (markers.indexOf("s") < 0 && !/(me*){3}/.exec(markers))
        markers = markers.replace(/m/g, "t");
    if (Utils_1.matchStart(markers, /[te]*f/))
        return result;
    var inlineReplyMatch;
    var inlineReplyRegexp = /me*(te*)+m/g;
    while (inlineReplyMatch = inlineReplyRegexp.exec(markers)) {
        var links = lines[inlineReplyMatch.index].match(Regexp_1.ParenthesisLinkRegexp)
            || Utils_1.matchStart(lines[inlineReplyMatch.index + 1].trim(), Regexp_1.ParenthesisLinkRegexp);
        if (!links)
            return result;
        inlineReplyRegexp.lastIndex--;
    }
    var quotation = markers.match("(se*)+((t|f)+e*)+");
    if (quotation) {
        result.wereLinesDeleted = true;
        result.firstDeletedLine = quotation.index;
        result.lastDeletedLine = lines.length - 1;
        result.lastMessageLines = lines.slice(0, quotation.index);
        return result;
    }
    quotation = markers.match(Regexp_1.QuotationRegexp)
        || markers.match(Regexp_1.EmptyQuotationRegexp);
    if (quotation) {
        var firstGroupStart = quotation.index + quotation[0].indexOf(quotation[1]);
        var firstGroupEnd = firstGroupStart + quotation[1].length;
        result.wereLinesDeleted = true;
        result.firstDeletedLine = firstGroupStart;
        result.lastDeletedLine = firstGroupEnd - 1;
        result.lastMessageLines = lines.slice(0, firstGroupStart).concat(lines.slice(firstGroupEnd));
        return result;
    }
    return result;
}
exports.processMarkedLines = processMarkedLines;
function postProcess(messageBody) {
    return messageBody.replace(new RegExp(Regexp_1.NormalizedLinkRegexp.source, "g"), "<$1>").trim();
}
function isSplitter(src) {
    for (var _i = 0, SplitterRegexps_1 = Regexp_1.SplitterRegexps; _i < SplitterRegexps_1.length; _i++) {
        var pattern = SplitterRegexps_1[_i];
        var match = Utils_1.matchStart(src, pattern);
        if (match)
            return match;
    }
}
function loadHtmlAndFix(src) {
    var document = Cheerio.load(src);
    document.root().find("*").contents().filter(function (n, e) { return e.type === "directive" && e.name[0] === "!"; }).remove();
    return document;
}
//# sourceMappingURL=Quotations.js.map