"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var XPath = require("xpath");
var Constants_1 = require("./Constants");
var Regexp_1 = require("./Regexp");
function findDelimiter(messageBody) {
    var match = Regexp_1.DelimiterRegexp.exec(messageBody);
    return match ? match[0] : "\n";
}
exports.findDelimiter = findDelimiter;
;
function splitLines(str) {
    return str.split(/\r?\n/);
}
exports.splitLines = splitLines;
;
function matchStart(str, regexp) {
    var match = str.match(regexp);
    return !match || match.index > 0 ? null : match;
}
exports.matchStart = matchStart;
;
function elementToText(element, _a) {
    var ignoreBlockTags = (_a === void 0 ? {} : _a).ignoreBlockTags;
    var styleNodes = XPath.select("//style", element);
    for (var _i = 0, styleNodes_1 = styleNodes; _i < styleNodes_1.length; _i++) {
        var styleNode = styleNodes_1[_i];
        styleNode.parentNode.removeChild(styleNode);
    }
    var commentNodes = XPath.select("//comment()", element);
    for (var _b = 0, commentNodes_1 = commentNodes; _b < commentNodes_1.length; _b++) {
        var commentNode = commentNodes_1[_b];
        commentNode.parentNode.removeChild(commentNode);
    }
    var text = "";
    var allNodes = XPath.select("//*", element);
    for (var _c = 0, allNodes_1 = allNodes; _c < allNodes_1.length; _c++) {
        var node = allNodes_1[_c];
        var nodeText = extractTextFromNode(node, text);
        if (nodeText.length > 1) {
            var nodeName = node.nodeName.toLowerCase();
            if (!ignoreBlockTags && Constants_1.BlockTags.includes(nodeName))
                text += "\n";
            if (node.nodeName.toLowerCase() === "li")
                text += "  * ";
            text += nodeText + " ";
            var href = node.attributes.getNamedItem("href");
            if (href)
                text += "(" + href + ") ";
        }
    }
    return removeExcessiveNewlines(text);
}
exports.elementToText = elementToText;
;
function extractTextFromNode(node, text) {
    var nodeValue = (node.nodeValue || (node.firstChild && node.firstChild.nodeType === Constants_1.NodeTypes.TEXT_NODE && node.firstChild.nodeValue) || '').trim();
    var sibillingValue = ((node.nextSibling && node.nextSibling.nodeType === Constants_1.NodeTypes.TEXT_NODE && node.nextSibling.nodeValue) || '').trim();
    if (Constants_1.HardbreakTags.indexOf(node.nodeName.toLowerCase()) >= 0
        && text && text[text.length - 1] !== "\n")
        nodeValue += "\n";
    var nodeText = nodeValue + sibillingValue;
    return nodeText.replace('\\n', '\n');
}
function normalizeHtmlDocument(document) {
    var matchStart = document.match(/^(\s*<html[^>]*>)?(\s*<head[^>]*>.*<\/head>)?(\s*<body[^>]*>)?/im);
    var matchEnd = document.match(/(<\/body>\s*)?(<\/html>\s*)?$/im);
    if (!matchStart[3])
        document = matchStart[0].trim() +
            "<body>" +
            document.slice(matchStart[0].length);
    if (!matchStart[1])
        document = "<html>" + document;
    if (!matchEnd[1])
        document = (matchEnd[0] ? document.slice(0, -matchEnd[0].length) : document) +
            "</body>" +
            matchEnd[0].trim();
    if (!matchEnd[2])
        document = document + "</html>";
    return document;
}
exports.normalizeHtmlDocument = normalizeHtmlDocument;
;
function removeExcessiveNewlines(src) {
    return src.replace(/\n{2,10}/g, "\n\n").trim();
}
//# sourceMappingURL=Utils.js.map